@@ -1,3 +1,6 @@
+// SPDX-FileCopyrightText: 2023 The Pion community <https://pion.ly>
+// SPDX-License-Identifier: MIT
+
 package sctp
 
 import (
@@ -22,15 +25,11 @@ func TestChunkForwardTSN_Success(t *testing.T) {
 	for i, tc := range tt {
 		actual := &chunkForwardTSN{}
 		err := actual.unmarshal(tc.binary)
-		if err != nil {
-			t.Fatalf("failed to unmarshal #%d: %v", i, err)
-		}
+		assert.NoErrorf(t, err, "failed to unmarshal #%d", i)
 
 		b, err := actual.marshal()
-		if err != nil {
-			t.Fatalf("failed to marshal: %v", err)
-		}
-		assert.Equal(t, tc.binary, b, "test %d not equal", i)
+		assert.NoError(t, err)
+		assert.Equalf(t, tc.binary, b, "test %d not equal", i)
 	}
 }
 
@@ -47,8 +46,6 @@ func TestChunkForwardTSNUnmarshal_Failure(t *testing.T) {
 	for i, tc := range tt {
 		actual := &chunkForwardTSN{}
 		err := actual.unmarshal(tc.binary)
-		if err == nil {
-			t.Errorf("expected unmarshal #%d: '%s' to fail.", i, tc.name)
-		}
+		assert.Errorf(t, err, "expected unmarshal #%d: '%s' to fail.", i, tc.name)
 	}
 }
