@@ -1,3 +1,6 @@
+// SPDX-FileCopyrightText: 2023 The Pion community <https://pion.ly>
+// SPDX-License-Identifier: MIT
+
 package sctp // nolint:dupl
 
 import (
@@ -10,21 +13,21 @@ chunkInitAck represents an SCTP Chunk of type INIT ACK
 
 See chunkInitCommon for the fixed headers
 
-Variable Parameters                  Status     Type Value
--------------------------------------------------------------
-State Cookie                        Mandatory   7
-IPv4 IP (Note 1)               Optional    5
-IPv6 IP (Note 1)               Optional    6
-Unrecognized Parameter              Optional    8
-Reserved for ECN Capable (Note 2)   Optional    32768 (0x8000)
-Host Name IP (Note 3)          Optional    11<Paste>
+	Variable Parameters                  Status     Type Value
+	-------------------------------------------------------------
+	State Cookie                        Mandatory   7
+	IPv4 IP (Note 1)               		Optional    5
+	IPv6 IP (Note 1)               		Optional    6
+	Unrecognized Parameter              Optional    8
+	Reserved for ECN Capable (Note 2)   Optional    32768 (0x8000)
+	Host Name IP (Note 3)          		Optional    11<Paste>
 */
 type chunkInitAck struct {
 	chunkHeader
 	chunkInitCommon
 }
 
-// Init ack chunk errors
+// Init ack chunk errors.
 var (
 	ErrChunkTypeNotInitAck              = errors.New("ChunkType is not of type INIT ACK")
 	ErrChunkNotLongEnoughForParams      = errors.New("chunk Value isn't long enough for mandatory parameters exp")
@@ -56,7 +59,7 @@ func (i *chunkInitAck) unmarshal(raw []byte) error {
 	}
 
 	if err := i.chunkInitCommon.unmarshal(i.raw); err != nil {
-		return fmt.Errorf("%w: %v", ErrInitAckUnmarshalFailed, err)
+		return fmt.Errorf("%w: %v", ErrInitAckUnmarshalFailed, err) //nolint:errorlint
 	}
 
 	return nil
@@ -65,11 +68,12 @@ func (i *chunkInitAck) unmarshal(raw []byte) error {
 func (i *chunkInitAck) marshal() ([]byte, error) {
 	initShared, err := i.chunkInitCommon.marshal()
 	if err != nil {
-		return nil, fmt.Errorf("%w: %v", ErrInitCommonDataMarshalFailed, err)
+		return nil, fmt.Errorf("%w: %v", ErrInitCommonDataMarshalFailed, err) //nolint:errorlint
 	}
 
 	i.chunkHeader.typ = ctInitAck
 	i.chunkHeader.raw = initShared
+
 	return i.chunkHeader.marshal()
 }
 
@@ -88,6 +92,7 @@ func (i *chunkInitAck) check() (abort bool, err error) {
 	// purpose.
 	if i.initiateTag == 0 {
 		abort = true
+
 		return abort, ErrChunkTypeInitAckInitateTagZero
 	}
 
@@ -103,6 +108,7 @@ func (i *chunkInitAck) check() (abort bool, err error) {
 	// destroy the association discarding its TCB.
 	if i.numInboundStreams == 0 {
 		abort = true
+
 		return abort, ErrInitAckInboundStreamRequestZero
 	}
 
@@ -116,6 +122,7 @@ func (i *chunkInitAck) check() (abort bool, err error) {
 
 	if i.numOutboundStreams == 0 {
 		abort = true
+
 		return abort, ErrInitAckOutboundStreamRequestZero
 	}
 
@@ -125,13 +132,14 @@ func (i *chunkInitAck) check() (abort bool, err error) {
 	// ACK.
 	if i.advertisedReceiverWindowCredit < 1500 {
 		abort = true
+
 		return abort, ErrInitAckAdvertisedReceiver1500
 	}
 
 	return false, nil
 }
 
-// String makes chunkInitAck printable
+// String makes chunkInitAck printable.
 func (i *chunkInitAck) String() string {
 	return fmt.Sprintf("%s\n%s", i.chunkHeader, i.chunkInitCommon)
 }
