@@ -1,3 +1,6 @@
+// SPDX-FileCopyrightText: 2023 The Pion community <https://pion.ly>
+// SPDX-License-Identifier: MIT
+
 package sctp // nolint:dupl
 
 import (
@@ -19,9 +22,10 @@ func TestParamForwardTSNSupported_Success(t *testing.T) {
 			testParamForwardTSNSupported(),
 			&paramForwardTSNSupported{
 				paramHeader: paramHeader{
-					typ: forwardTSNSupp,
-					len: 4,
-					raw: []byte{},
+					typ:                forwardTSNSupp,
+					len:                4,
+					unrecognizedAction: paramHeaderUnrecognizedActionSkipAndReport,
+					raw:                []byte{},
 				},
 			},
 		},
@@ -30,14 +34,11 @@ func TestParamForwardTSNSupported_Success(t *testing.T) {
 	for i, tc := range tt {
 		actual := &paramForwardTSNSupported{}
 		_, err := actual.unmarshal(tc.binary)
-		if err != nil {
-			t.Fatalf("failed to unmarshal #%d: %v", i, err)
-		}
+		assert.NoErrorf(t, err, "failed to unmarshal #%d", i)
 		assert.Equal(t, tc.parsed, actual)
+
 		b, err := actual.marshal()
-		if err != nil {
-			t.Fatalf("failed to marshal: %v", err)
-		}
+		assert.NoErrorf(t, err, "failed to unmarshal #%d", i)
 		assert.Equal(t, tc.binary, b)
 	}
 }
@@ -53,8 +54,6 @@ func TestParamForwardTSNSupported_Failure(t *testing.T) {
 	for i, tc := range tt {
 		actual := &paramForwardTSNSupported{}
 		_, err := actual.unmarshal(tc.binary)
-		if err == nil {
-			t.Errorf("expected unmarshal #%d: '%s' to fail.", i, tc.name)
-		}
+		assert.Errorf(t, err, "expected unmarshal #%d: '%s' to fail.", i, tc.name)
 	}
 }
