@@ -1,18 +1,23 @@
+// SPDX-FileCopyrightText: 2023 The Pion community <https://pion.ly>
+// SPDX-License-Identifier: MIT
+
 package sctp
 
 import (
 	"bytes"
+	crand "crypto/rand"
 	"fmt"
-	"math/rand"
 	"net"
 	"reflect"
+	"sync/atomic"
 	"testing"
 	"time"
 
 	"github.com/pion/logging"
-	"github.com/pion/transport/test"
-	"github.com/pion/transport/vnet"
+	"github.com/pion/transport/v3/test"
+	"github.com/pion/transport/v3/vnet"
 	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
 )
 
 type vNetEnvConfig struct {
@@ -31,8 +36,6 @@ type vNetEnv struct {
 	numToDropCookieAck  int
 }
 
-var errSCTPPacketParse = fmt.Errorf("unable to parse SCTP packet")
-
 func (venv *vNetEnv) dropNextDataChunk(numToDrop int) {
 	venv.numToDropData = numToDrop
 }
@@ -49,7 +52,8 @@ func (venv *vNetEnv) dropNextCookieAckChunk(numToDrop int) {
 	venv.numToDropCookieAck = numToDrop
 }
 
-func buildVNetEnv(cfg *vNetEnvConfig) (*vNetEnv, error) {
+func buildVNetEnv(t *testing.T, cfg *vNetEnvConfig) (*vNetEnv, error) { //nolint:cyclop
+	t.Helper()
 	log := cfg.log
 
 	var venv *vNetEnv
@@ -73,9 +77,7 @@ func buildVNetEnv(cfg *vNetEnvConfig) (*vNetEnv, error) {
 		return func(c vnet.Chunk) bool {
 			var toDrop bool
 			p := &packet{}
-			if err2 := p.unmarshal(c.UserData()); err2 != nil {
-				panic(errSCTPPacketParse)
-			}
+			assert.NoError(t, p.unmarshal(true, c.UserData()))
 
 		loop:
 			for i := 0; i < len(p.chunks); i++ {
@@ -91,6 +93,7 @@ func buildVNetEnv(cfg *vNetEnvConfig) (*vNetEnv, error) {
 							toDrop = true
 							venv.numToDropData--
 							log.Infof("Chunk filter:  drop TSN %d", tsn)
+
 							break loop
 						}
 					}
@@ -99,6 +102,7 @@ func buildVNetEnv(cfg *vNetEnvConfig) (*vNetEnv, error) {
 						toDrop = true
 						venv.numToDropReconfig--
 						log.Infof("Chunk filter:  drop RECONFIG %s", chunk.String())
+
 						break loop
 					}
 				case *chunkCookieEcho:
@@ -106,6 +110,7 @@ func buildVNetEnv(cfg *vNetEnvConfig) (*vNetEnv, error) {
 						toDrop = true
 						venv.numToDropCookieEcho--
 						log.Infof("Chunk filter:  drop %s", chunk.String())
+
 						break loop
 					}
 				case *chunkCookieAck:
@@ -113,26 +118,36 @@ func buildVNetEnv(cfg *vNetEnvConfig) (*vNetEnv, error) {
 						toDrop = true
 						venv.numToDropCookieAck--
 						log.Infof("Chunk filter:  drop %s", chunk.String())
+
 						break loop
 					}
 				}
 			}
+
 			return !toDrop
 		}
 	}
 	wan.AddChunkFilter(tsnAutoLockOnFilter())
 
-	net0 := vnet.NewNet(&vnet.NetConfig{
+	net0, err := vnet.NewNet(&vnet.NetConfig{
 		StaticIPs: []string{serverIP},
 	})
+	if err != nil {
+		return nil, err
+	}
+
 	err = wan.AddNet(net0)
 	if err != nil {
 		return nil, err
 	}
 
-	net1 := vnet.NewNet(&vnet.NetConfig{
+	net1, err := vnet.NewNet(&vnet.NetConfig{
 		StaticIPs: []string{clientIP},
 	})
+	if err != nil {
+		return nil, err
+	}
+
 	err = wan.AddNet(net1)
 	if err != nil {
 		return nil, err
@@ -152,21 +167,19 @@ func buildVNetEnv(cfg *vNetEnvConfig) (*vNetEnv, error) {
 	return venv, nil
 }
 
-func testRwndFull(t *testing.T, unordered bool) {
+func testRwndFull(t *testing.T, unordered bool) { //nolint:cyclop
+	t.Helper()
+
 	loggerFactory := logging.NewDefaultLoggerFactory()
 	log := loggerFactory.NewLogger("test")
 
-	venv, err := buildVNetEnv(&vNetEnvConfig{
+	venv, err := buildVNetEnv(t, &vNetEnvConfig{
 		minDelay:      200 * time.Millisecond,
 		loggerFactory: loggerFactory,
 		log:           log,
 	})
-	if !assert.NoError(t, err, "should succeed") {
-		return
-	}
-	if !assert.NotNil(t, venv, "should not be nil") {
-		return
-	}
+	require.NoError(t, err, "should succeed")
+	require.NotNil(t, venv, "should not be nil")
 	defer venv.wan.Stop() // nolint:errcheck
 
 	serverHandshakeDone := make(chan struct{})
@@ -185,14 +198,16 @@ func testRwndFull(t *testing.T, unordered bool) {
 	maxReceiveBufferSize := uint32(64 * 1024)
 	msgSize := int(float32(maxReceiveBufferSize)/2) + int(initialMTU)
 	msg := make([]byte, msgSize)
-	rand.Read(msg) // nolint:errcheck,gosec
+	n, err := crand.Read(msg)
+	require.NoError(t, err, "failed to read random bytes")
+	require.Equal(t, len(msg), n, "short random read")
 
 	go func() {
 		defer close(serverShutDown)
 		// connected UDP conn for server
 		conn, err := venv.net0.DialUDP("udp4",
-			&net.UDPAddr{IP: net.ParseIP("1.1.1.1"), Port: 5000},
-			&net.UDPAddr{IP: net.ParseIP("2.2.2.2"), Port: 5000},
+			&net.UDPAddr{IP: net.ParseIP("1.1.1.1"), Port: defaultSCTPSrcDstPort},
+			&net.UDPAddr{IP: net.ParseIP("2.2.2.2"), Port: defaultSCTPSrcDstPort},
 		)
 		if !assert.NoError(t, err, "should succeed") {
 			return
@@ -254,7 +269,7 @@ func testRwndFull(t *testing.T, unordered bool) {
 				return
 			}
 			log.Infof("server read %d bytes", n)
-			assert.True(t, reflect.DeepEqual(msg, buf[:n]), "msg %d should match", i)
+			assert.Truef(t, reflect.DeepEqual(msg, buf[:n]), "msg %d should match", i)
 		}
 
 		close(serverReadAll)
@@ -266,8 +281,8 @@ func testRwndFull(t *testing.T, unordered bool) {
 		defer close(clientShutDown)
 		// connected UDP conn for client
 		conn, err := venv.net1.DialUDP("udp4",
-			&net.UDPAddr{IP: net.ParseIP("2.2.2.2"), Port: 5000},
-			&net.UDPAddr{IP: net.ParseIP("1.1.1.1"), Port: 5000},
+			&net.UDPAddr{IP: net.ParseIP("2.2.2.2"), Port: defaultSCTPSrcDstPort},
+			&net.UDPAddr{IP: net.ParseIP("1.1.1.1"), Port: defaultSCTPSrcDstPort},
 		)
 		if !assert.NoError(t, err, "should succeed") {
 			return
@@ -386,24 +401,22 @@ func TestRwndFull(t *testing.T) {
 	})
 }
 
-func TestStreamClose(t *testing.T) {
+func TestStreamClose(t *testing.T) { //nolint:cyclop
 	loopBackTest := func(t *testing.T, dropReconfigChunk bool) {
+		t.Helper()
+
 		lim := test.TimeOut(time.Second * 10)
 		defer lim.Stop()
 
 		loggerFactory := logging.NewDefaultLoggerFactory()
 		log := loggerFactory.NewLogger("test")
 
-		venv, err := buildVNetEnv(&vNetEnvConfig{
+		venv, err := buildVNetEnv(t, &vNetEnvConfig{
 			loggerFactory: loggerFactory,
 			log:           log,
 		})
-		if !assert.NoError(t, err, "should succeed") {
-			return
-		}
-		if !assert.NotNil(t, venv, "should not be nil") {
-			return
-		}
+		require.NoError(t, err, "should succeed")
+		require.NotNil(t, venv, "should not be nil")
 		defer venv.wan.Stop() // nolint:errcheck
 
 		clientShutDown := make(chan struct{})
@@ -424,8 +437,8 @@ func TestStreamClose(t *testing.T) {
 			defer close(serverShutDown)
 			// connected UDP conn for server
 			conn, innerErr := venv.net0.DialUDP("udp4",
-				&net.UDPAddr{IP: net.ParseIP("1.1.1.1"), Port: 5000},
-				&net.UDPAddr{IP: net.ParseIP("2.2.2.2"), Port: 5000},
+				&net.UDPAddr{IP: net.ParseIP("1.1.1.1"), Port: defaultSCTPSrcDstPort},
+				&net.UDPAddr{IP: net.ParseIP("2.2.2.2"), Port: defaultSCTPSrcDstPort},
 			)
 			if !assert.NoError(t, innerErr, "should succeed") {
 				return
@@ -457,6 +470,7 @@ func TestStreamClose(t *testing.T) {
 					log.Infof("server: Read returned %v", errRead)
 					_ = stream.Close() // nolint:errcheck
 					assert.Equal(t, StreamStateClosed, stream.State())
+
 					break
 				}
 
@@ -474,8 +488,8 @@ func TestStreamClose(t *testing.T) {
 
 		// connected UDP conn for client
 		conn, err := venv.net1.DialUDP("udp4",
-			&net.UDPAddr{IP: net.ParseIP("2.2.2.2"), Port: 5000},
-			&net.UDPAddr{IP: net.ParseIP("1.1.1.1"), Port: 5000},
+			&net.UDPAddr{IP: net.ParseIP("2.2.2.2"), Port: defaultSCTPSrcDstPort},
+			&net.UDPAddr{IP: net.ParseIP("1.1.1.1"), Port: defaultSCTPSrcDstPort},
 		)
 		if !assert.NoError(t, err, "should succeed") {
 			return
@@ -510,6 +524,7 @@ func TestStreamClose(t *testing.T) {
 				if err2 != nil {
 					log.Infof("client: Read returned %v", err2)
 					assert.Equal(t, StreamStateClosed, stream.State())
+
 					break
 				}
 
@@ -570,7 +585,7 @@ func TestStreamClose(t *testing.T) {
 // and confirmes the fix.
 // To reproduce the case mentioned above:
 // * Use simultaneous-open (SCTP)
-// * Drop both of the first COOKIE-ECHO and COOKIE-ACK
+// * Drop both of the first COOKIE-ECHO and COOKIE-ACK.
 func TestCookieEchoRetransmission(t *testing.T) {
 	lim := test.TimeOut(time.Second * 10)
 	defer lim.Stop()
@@ -578,17 +593,13 @@ func TestCookieEchoRetransmission(t *testing.T) {
 	loggerFactory := logging.NewDefaultLoggerFactory()
 	log := loggerFactory.NewLogger("test")
 
-	venv, err := buildVNetEnv(&vNetEnvConfig{
+	venv, err := buildVNetEnv(t, &vNetEnvConfig{
 		minDelay:      200 * time.Millisecond,
 		loggerFactory: loggerFactory,
 		log:           log,
 	})
-	if !assert.NoError(t, err, "should succeed") {
-		return
-	}
-	if !assert.NotNil(t, venv, "should not be nil") {
-		return
-	}
+	require.NoError(t, err, "should succeed")
+	require.NotNil(t, venv, "should not be nil")
 	defer venv.wan.Stop() // nolint:errcheck
 
 	// To cause the cookie echo retransmission, both COOKIE-ECHO
@@ -609,8 +620,8 @@ func TestCookieEchoRetransmission(t *testing.T) {
 		defer close(serverShutDown)
 		// connected UDP conn for server
 		conn, err := venv.net0.DialUDP("udp4",
-			&net.UDPAddr{IP: net.ParseIP("1.1.1.1"), Port: 5000},
-			&net.UDPAddr{IP: net.ParseIP("2.2.2.2"), Port: 5000},
+			&net.UDPAddr{IP: net.ParseIP("1.1.1.1"), Port: defaultSCTPSrcDstPort},
+			&net.UDPAddr{IP: net.ParseIP("2.2.2.2"), Port: defaultSCTPSrcDstPort},
 		)
 		if !assert.NoError(t, err, "should succeed") {
 			return
@@ -639,8 +650,8 @@ func TestCookieEchoRetransmission(t *testing.T) {
 		defer close(clientShutDown)
 		// connected UDP conn for client
 		conn, err := venv.net1.DialUDP("udp4",
-			&net.UDPAddr{IP: net.ParseIP("2.2.2.2"), Port: 5000},
-			&net.UDPAddr{IP: net.ParseIP("1.1.1.1"), Port: 5000},
+			&net.UDPAddr{IP: net.ParseIP("2.2.2.2"), Port: defaultSCTPSrcDstPort},
+			&net.UDPAddr{IP: net.ParseIP("1.1.1.1"), Port: defaultSCTPSrcDstPort},
 		)
 		if !assert.NoError(t, err, "should succeed") {
 			return
@@ -677,3 +688,269 @@ func TestCookieEchoRetransmission(t *testing.T) {
 	<-serverShutDown
 	log.Info("all done")
 }
+
+// Simulate an RTT switch (high -> low) by delaying early DATA, then disabling delay so
+// later DATA arrives before earlier DATA. Under a RACK regression, rackMinRTT would never increases,
+// causing reoWnd to be too small and marking packets sent at high RTT as spuriously lost.
+func TestRACK_RTTSwitch_Reordering_NoDrop(t *testing.T) { //nolint:gocyclo,cyclop,maintidx
+	lim := test.TimeOut(15 * time.Second)
+	defer lim.Stop()
+
+	loggerFactory := logging.NewDefaultLoggerFactory()
+	log := loggerFactory.NewLogger("test-rack-rtt-switch")
+
+	venv, err := buildVNetEnv(t, &vNetEnvConfig{
+		minDelay:      0,
+		loggerFactory: loggerFactory,
+		log:           log,
+	})
+	require.NoError(t, err)
+	require.NotNil(t, venv)
+
+	defer venv.wan.Stop() // nolint:errcheck
+
+	var delayOn atomic.Value
+	delayOn.Store(true)
+	venv.wan.AddChunkFilter(func(c vnet.Chunk) bool {
+		p := &packet{}
+		if err := p.unmarshal(true, c.UserData()); err != nil {
+			return true
+		}
+		v := delayOn.Load()
+		if val, ok := v.(bool); ok && !val {
+			return true
+		}
+		for i := 0; i < len(p.chunks); i++ {
+			if _, ok := p.chunks[i].(*chunkPayloadData); ok {
+				time.Sleep(100 * time.Millisecond)
+
+				break
+			}
+		}
+
+		return true
+	})
+
+	const (
+		numMessages = 40
+		messageSize = 256
+	)
+
+	makeMessages := func() [][]byte {
+		msgs := make([][]byte, numMessages)
+		for i := 0; i < numMessages; i++ {
+			b := bytes.Repeat([]byte{byte(i % 251)}, messageSize)
+			msgs[i] = b
+		}
+
+		return msgs
+	}
+
+	type statsResult struct {
+		fr uint64
+		ok bool
+	}
+
+	errCh := make(chan error, 16)
+	clientDone := make(chan struct{})
+	serverDone := make(chan struct{})
+	clientStatsCh := make(chan statsResult, 1)
+	serverStatsCh := make(chan statsResult, 1)
+
+	go func() {
+		defer close(serverDone)
+
+		fail := func(e error) {
+			if e != nil {
+				errCh <- e
+			}
+		}
+
+		conn, err := venv.net0.DialUDP("udp4",
+			&net.UDPAddr{IP: net.ParseIP("1.1.1.1"), Port: defaultSCTPSrcDstPort},
+			&net.UDPAddr{IP: net.ParseIP("2.2.2.2"), Port: defaultSCTPSrcDstPort},
+		)
+		if err != nil {
+			fail(fmt.Errorf("server DialUDP: %w", err))
+			serverStatsCh <- statsResult{ok: false}
+
+			return
+		}
+
+		defer conn.Close() // nolint:errcheck
+
+		assoc, err := Server(Config{
+			NetConn:       conn,
+			LoggerFactory: loggerFactory,
+		})
+		if err != nil {
+			fail(fmt.Errorf("server assoc: %w", err))
+			serverStatsCh <- statsResult{ok: false}
+
+			return
+		}
+
+		defer func() {
+			var fr uint64
+			if assoc != nil {
+				fr = assoc.stats.getNumFastRetrans()
+			}
+			serverStatsCh <- statsResult{fr: fr, ok: assoc != nil}
+			_ = assoc.Close()
+		}()
+
+		stream, err := assoc.AcceptStream()
+		if err != nil {
+			fail(fmt.Errorf("server AcceptStream: %w", err))
+
+			return
+		}
+		defer stream.Close() // nolint:errcheck
+		stream.SetReliabilityParams(false, ReliabilityTypeReliable, 0)
+
+		buf := make([]byte, 1500)
+		for {
+			_ = stream.SetReadDeadline(time.Now().Add(500 * time.Millisecond))
+			n, rerr := stream.Read(buf)
+			if rerr != nil {
+				return
+			}
+			if n > 0 {
+				_, _ = stream.Write(buf[:n])
+			}
+		}
+	}()
+
+	go func() {
+		defer close(clientDone)
+
+		fail := func(e error) {
+			if e != nil {
+				errCh <- e
+			}
+		}
+
+		conn, err := venv.net1.DialUDP("udp4",
+			&net.UDPAddr{IP: net.ParseIP("2.2.2.2"), Port: defaultSCTPSrcDstPort},
+			&net.UDPAddr{IP: net.ParseIP("1.1.1.1"), Port: defaultSCTPSrcDstPort},
+		)
+		if err != nil {
+			fail(fmt.Errorf("client DialUDP: %w", err))
+			clientStatsCh <- statsResult{ok: false}
+
+			return
+		}
+		defer conn.Close() // nolint:errcheck
+
+		assoc, err := Client(Config{
+			NetConn:       conn,
+			LoggerFactory: loggerFactory,
+		})
+		if err != nil {
+			fail(fmt.Errorf("client assoc: %w", err))
+			clientStatsCh <- statsResult{ok: false}
+
+			return
+		}
+
+		defer func() {
+			var fr uint64
+			if assoc != nil {
+				fr = assoc.stats.getNumFastRetrans()
+			}
+			clientStatsCh <- statsResult{fr: fr, ok: assoc != nil}
+			_ = assoc.Close()
+		}()
+
+		stream, err := assoc.OpenStream(777, PayloadTypeWebRTCBinary)
+		if err != nil {
+			fail(fmt.Errorf("client OpenStream: %w", err))
+
+			return
+		}
+		defer stream.Close() // nolint:errcheck
+		stream.SetReliabilityParams(false, ReliabilityTypeReliable, 0)
+
+		msgs := makeMessages()
+
+		// phase 1: high-RTT emulation we send 25 messages and drop a DATA chunk for one time.
+		delayOn.Store(true)
+		venv.dropNextDataChunk(1)
+		for i := 0; i < 25; i++ {
+			if _, werr := stream.Write(msgs[i]); werr != nil {
+				fail(fmt.Errorf("client write phase1 i=%d: %w", i, werr))
+
+				return
+			}
+		}
+
+		// phase 2 we switch to low-RTT, newer datea should arrive before older.
+		delayOn.Store(false)
+		for i := 25; i < numMessages; i++ {
+			if _, werr := stream.Write(msgs[i]); werr != nil {
+				fail(fmt.Errorf("client write phase2 i=%d: %w", i, werr))
+
+				return
+			}
+		}
+
+		seen := make(map[byte]bool, numMessages)
+		buf := make([]byte, 4096)
+		deadline := time.Now().Add(15 * time.Second)
+
+		for len(seen) < numMessages && time.Now().Before(deadline) {
+			_ = stream.SetReadDeadline(time.Now().Add(250 * time.Millisecond))
+			n, rerr := stream.Read(buf)
+			if rerr != nil || n == 0 {
+				continue
+			}
+			if n < messageSize {
+				fail(fmt.Errorf("short echo read: got=%d want=%d", n, messageSize)) //nolint:err113
+
+				return
+			}
+			id := buf[0]
+			if seen[id] {
+				// dups are harmless, keep reading
+				continue
+			}
+
+			expected := bytes.Repeat([]byte{id}, messageSize)
+			if !bytes.Equal(buf[:messageSize], expected) {
+				fail(fmt.Errorf("payload mismatch for id=%d", int(id))) //nolint:err113
+
+				return
+			}
+			seen[id] = true
+		}
+
+		if len(seen) != numMessages {
+			fail(fmt.Errorf("missing echoes: got=%d want=%d", len(seen), numMessages)) //nolint:err113
+
+			return
+		}
+	}()
+
+	<-clientDone
+	<-serverDone
+
+	// drain and assert errors, well if any :)
+	close(errCh)
+	for e := range errCh {
+		assert.NoError(t, e)
+	}
+
+	// check FR stats reported.
+	cs := <-clientStatsCh
+	ss := <-serverStatsCh
+
+	if assert.True(t, cs.ok, "client assoc/stats unavailable") {
+		assert.LessOrEqual(t, cs.fr, uint64(2),
+			"client fast retransmits should be low")
+	}
+
+	if assert.True(t, ss.ok, "server assoc/stats unavailable") {
+		assert.LessOrEqual(t, ss.fr, uint64(2),
+			"server fast retransmits should be low")
+	}
+}
